var browser, chrome, isFirefox;

if (typeof browser !== "undefined") {
  isFirefox = true;
} else if (typeof chrome !== "undefined") {
  isFirefox = false;
} else {
  throw "Unsupported browser! Change browser!";
}

var linkPortMap = {};
var idLinkMap = {};

setupListeners();

function setupListeners() {
  if (isFirefox) {
    browser.runtime.onConnect.addListener(handleConnected);
    browser.downloads.onChanged.addListener(handleDownloadChange);
  } else {
    chrome.runtime.onConnect.addListener(handleConnected);
    chrome.downloads.onChanged.addListener(handleDownloadChange);
  }
}

function handleConnected(port) {
  port.onMessage.addListener(function(message) {
    processDownload(port, message);
  });
}

function processDownload(port, message) {
  let path = "gallerymaker.net/" + message.galleryName + "/" + message.fileName;
  let url = message.url;

  linkPortMap[url] = port;

  download(url, path)
    .then(id => {
      idLinkMap[id] = url;
    })
    .catch(e => {
      console.log(e);
      postMessage("incorrect", url);
    });
}

function download(url, path) {
  if (isFirefox) {
    return browser.downloads.download({
      url: url,
      filename: path,
      conflictAction: "overwrite"
    });
  } else {
    return new Promise(function(resolve, reject) {
      try {
        chrome.downloads.download(
          {
            url: url,
            filename: path,
            conflictAction: "overwrite"
          },
          result => {
            resolve(result);
          }
        );
      } catch (e) {
        reject(e);
      }
    });
  }
}

function postMessage(type, url) {
  try {
    linkPortMap[url].postMessage({
      type: type,
      link: url
    });

    delete linkPortMap[url];
  } catch (e) {
    console.log(e);
  }
}

function handleDownloadChange(item) {
  let id = item.id;
  let url = idLinkMap[id];

  if (item.state && item.state.current === "complete") {
    postMessage("correct", url);

    delete idLinkMap[id];
  } else if (item.error) {
    postMessage("incorrect", url);

    delete idLinkMap[id];
  }
}
