(function() {
  if (typeof browser === "undefined" && typeof crome !== "undefined") {
    console.log("Unsupported browser! Galley Maker Extension will not work!");
    return false;
  }

  document.body.classList.add("extension-on");

  var browser = browser || chrome;

  let items = [];
  let startQuantity = 0;
  let correct = [];
  let incorrect = [];
  var connectPort;

  document.addEventListener("download-start", downloadStart);

  function downloadStart(e) {
    document.removeEventListener("download-start", downloadStart);
    setupConnection();

    let galleryData = JSON.parse(e.detail);

    setItemsList(galleryData);
    setStartQuantity();

    statusUpdate();
    sendOneItem();
  }

  function setupConnection() {
    connectPort = browser.runtime.connect();
    connectPort.onMessage.addListener(receiveAnswer);
  }

  function receiveAnswer(message) {
    if (message.type === "correct") {
      correct.push(message.link);
    } else {
      incorrect.push(message.link);
    }

    statusUpdate();
    sendOneItem();
  }

  function statusUpdate() {
    document.body.dataset.downloadAll = startQuantity;
    document.body.dataset.downloadOk = correct.length;
    document.body.dataset.downloadErr = incorrect.length;

    let event = new Event("download-progressbar-updated");
    document.dispatchEvent(event);
  }

  function setItemsList(galleryData) {
    for (let link of galleryData.list) {
      let parts = link.split("/");
      let fileName = parts[parts.length - 1];

      items.push({
        link: link,
        fileName: fileName,
        galleryName: galleryData.name
      });
    }
  }

  function setStartQuantity() {
    startQuantity = items.length;
  }

  function sendOneItem() {
    if (items.length > 0) {
      let item = items.shift();
      sendToDownload(item);
    }
  }

  function sendToDownload(item) {
    connectPort.postMessage({
      url: item.link,
      fileName: item.fileName,
      galleryName: item.galleryName
    });
  }
})();
